/*

Copyright (c) 2005-2015, University of Oxford.
All rights reserved.

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the University of Oxford nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef APSIMULATOR_HPP_
#define APSIMULATOR_HPP_

#include "Exception.hpp"
#include "AbstractCvodeCell.hpp"
#include "OutputFileHandler.hpp"

/**
 * General cell model generator.
 *
 * Cell model choice made in source code.
 */

class APSimulator
{
private:
	double mSigma;
	boost::shared_ptr<AbstractCvodeCell> mpModel;

	double mSamplingTimestep;
	double mStartTime;
	double mEndTime;
	double mEndTime_half;
	std::vector<double> mExperimentalTrace;
	std::vector<double> mExperimentalTimes;
	std::vector<double> mExptTimes1;
	std::vector<double> mExptTimes2;
	std::vector<double> mExptTrace1;
	std::vector<double> mExptTrace2;
	std::vector<double> mTestTrace;
	std::vector<double> mTestTrace1;
	std::vector<double> mTestTrace2;

	unsigned mModelChoice;
	unsigned mProtocolChoice;
	unsigned mDog;

	unsigned mNumberOfSingularities;
	int mSeed;
	int mFreezingIndex;
	std::vector<std::string> mParameterMetanames;

	bool mRunningToSteadyState;

	double mDataClampOn;
	double mDataClampOff;

public:
	APSimulator(unsigned rModelChoice, unsigned protocolChoice, unsigned rSeed, int rFreezingIndex, unsigned dog);
	~APSimulator();
    void SetNoiseModelParameter(const std::vector<double>& rTestParameters);
    double GetNoiseModelParameter();
    void Run(const std::vector<double>& rTestParameters);
    void AddNoiseToExperimentalTrace();
    double CalculateLogLikelihood();
    std::vector<double> ReturnTestSolution();
    std::vector<double> ReturnTestTimes();
    std::vector<double> ReturnExperimentalTrace();
    unsigned ReturnNumberOfSingularities();
    void GenerateExperimentalTrace();
    void ArchiveStateVariables();
    void LoadStateVariables();
    void RunToSteadyState();
    void LoadExperimentalTrace();
    std::vector<double> GetStateVariables();
    void SetInitialVoltageToData();
    double GetVariable(const std::string& rVariableName);
    void SetParameter(const std::string& rParameterName, double value);
    double GetParameter(const std::string& rParameterName);
};

#endif /*APSIMULATOR_HPP_*/
