import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import Normalize
import os
import sys
import shutil

plt.rc('xtick', labelsize=14)
plt.rc('ytick', labelsize=14)
plt.rc('xtick.major', size=10, width=2)
plt.rc('ytick.major', size=10, width=2)
plt.rc({'font.size': 18, 'text.usetex': True})



def AnalyseMCMC(model, protocol, c_seed, subset_indices=None, freezing_index=None, override=False, true_lines=True, burn=False):
    
    plt.close()

    if model == 1:
        parameters = ['g_{Na}', 'g_K', 'g_l',r'\sigma']
        fig_labels = ['G_{Na}', 'G_K', 'G_l',r'\sigma']
        true_parameters = [120, 36, 0.3, 0.25]
    elif model == 2:
        parameters = ['g_{Na}', 'g_{K1}', 'g_K', 'g_l',r'\sigma']
        fig_labels = ['G_{Na}', 'G_{K1}', 'G_K', 'gG_l',r'\sigma']
        true_parameters = [400, 0.015, 1.2, 0.075, 0.25]    
    elif model == 3:
        parameters = ['g_{Na}', 'g_{K1}', 'g_K', 'g_{CaL}',r'\sigma']
        fig_labels = ['G_{Na}', 'G_{K1}', 'G_K', 'G_{CaL}',r'\sigma']
        true_parameters = [0.04, 0.0035, 0.008, 9e-4, 0.25]
    elif model == 4:
        parameters = ['g_{Na}', 'g_K', 'g_{K1}',
                      'g_{CaL}', 'g_l', 'g_{Kp}',r'\sigma']
        fig_labels = ['G_{Na}', 'G_K', 'G_{K1}',
                      'G_{CaL}', 'G_l', 'G_{Kp}',r'\sigma']
        true_parameters = [23, 0.282, 0.6047,
                           0.09, 0.03921, 0.0183, 0.25]
    elif model == 5:
        parameters = ['g_{Na}', 'g_{CaL}', 'g_{K1}', 'g_{Kr}', 'g_{Ks}', 'g_{NaCa}',
                      'g_{to}', 'g_{bCa}', 'g_{bNa}', 'g_{pCa}', 'g_{pK}', 'P_{NaK}',r'\sigma']
        fig_labels = ['G_{Na}', 'G_{CaL}', 'G_{K1}', 'G_{Kr}', 'G_{Ks}', 'k_{NaCa}',
                      'G_{to}', 'G_{bCa}', 'G_{bNa}', 'G_{pCa}', 'G_{pK}', 'P_{NaK}',r'\sigma']
        true_parameters = [14.838, 0.000175, 5.405, 0.096, 0.245, 1000,
                           0.294, 0.000592, 0.00029, 0.825, 0.0146, 1.362,
                           0.25]
    elif model == 6:
        parameters = ['g_{Na}', 'g_{CaL}', 'g_{bK}', 'g_{K1}',
                      'g_{Kr}', 'g_{Ks}', 'g_{NaCa}', 'P_{NaK}',
                      'g_{to}', 'g_{bCa}', 'g_{bNa}', 'g_{pCa}', 'g_{NaL}',r'\sigma']
        fig_labels = ['G_{Na}', 'G_{CaL}', 'G_{bK}', 'G_{K1}',
                      'G_{Kr}', 'G_{Ks}', 'k_{NaCa}', 'P_{NaK}',
                      'G_{to}', 'G_{bCa}', 'G_{bNa}', 'G_{pCa}', 'G_{NaL}',r'\sigma']
        true_parameters = [75, 0.0001, 0.003, 0.1908,
                           0.046, 0.0034, 0.0008, 30,
                           0.02, 2.5e-8, 3.75e-10, 0.0005, 0.0075,
                           0.25]
    elif model==7:
        parameters = ['g_{Na}', 'g_{CaL}', 'g_{K1}', 'g_{Kr}', 'g_{Ks}', 'g_{NaCa}',
                          'g_{to}', 'g_{bCa}', 'g_{bNa}', 'g_{pCa}', 'g_{pK}', 'P_{NaK}',
                          r'\sigma']
        fig_labels = ['G_{Na}', 'G_{CaL}', 'G_{K1}', 'G_{Kr}', 'G_{Ks}', 'k_{NaCa}',
                      'G_{to}', 'G_{bCa}', 'G_{bNa}', 'G_{pCa}', 'G_{pK}', 'P_{NaK}',r'\sigma']
        true_parameters = [14.838, 0.0000398, 5.405, 0.153, 0.392, 1000,
                   0.294, 0.000592, 0.00029,0.1238, 0.0146, 2.724,
                   0.25]
        
        
        
    number_of_parameters = len(parameters)

    if protocol != 3:
        chain_directory = 'projects/Jmcc_UQ/results/MCMC/model_'+str(model)+'/protocol_'+str(protocol)+'/'
        chain_file = chain_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_MCMC.txt'
        output_directory = chain_directory+'analysis/'
        log_file = output_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_MCMC.log'
    elif freezing_index == -1:
        chain_directory = 'projects/Jmcc_UQ/results/MCMC/model_'+str(model)+'/protocol_'+str(protocol)+'/no_freezing/'
        chain_file = chain_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_no_freezing_MCMC.txt'
        output_directory = chain_directory+'analysis/'
        log_file = output_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_no_freezing_MCMC.log'
    else:
        chain_directory = 'projects/Jmcc_UQ/results/MCMC/model_'+str(model)+'/protocol_'+str(protocol)+'/'+g_parameters[freezing_index]+'/'
        chain_file = chain_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_'+g_parameters[freezing_index]+'_MCMC.txt'
        output_directory = chain_directory+'analysis/'
        log_file = output_directory+'model_'+str(model)+'_protocol_'+str(protocol)+'_'+g_parameters[freezing_index]+'_MCMC.log'

    
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
            

    print chain_file
    if not os.path.isfile(chain_file): # Check if there is any MCMC at all, otherwise stop
        if (not freezing_index>=0):
            print "No MCMC for model",model,"protocol",protocol,"c_seed",c_seed
        elif (not freezing_index<0):
            print "No MCMC for model",model,"protocol",protocol,"c_seed",c_seed,parameters[freezing_index]
        if os.path.exists(directory):
            print "Deleting directory:", directory
            shutil.rmtree(directory)
        return None
    
    if os.path.isfile(log_file):
        with open(log_file,'r') as f:
            for line in f:
                if "Total timesteps" in line:
                    iterations_analysed = int(line.split(' = ')[1])
                    break
    else:
        iterations_analysed = 0

    try:
        iterations_computed,final_loga = np.loadtxt(chain_file,usecols=[5+3*number_of_parameters-1,
                                                    3+number_of_parameters-1])[-1,:]
        iterations_computed = int(iterations_computed)/10 + 1
    except:
        print "No values stored, i.e. MCMC not actually run (or at least not saved)."
        return None
                


    if ( ( iterations_computed == iterations_analysed ) and ( not override ) ):
        if (not freezing_index>=0):
            print "No need to update the graphs for model",model,"protocol",protocol,"c_seed",c_seed
        else:
            print "No need to update the graphs for model",model,"protocol",protocol,"c_seed",c_seed,parameters[freezing_index]
        return None
    elif override:
        print "Override is True, so will proceed assuming the entire chain was run again."

        
    images_output_directory_pdf = output_directory+'images/pdf/'
    images_output_directory_png = output_directory+'images/png/'
    images_output_directory_eps = output_directory+'images/eps/'
    images_output_directories = [images_output_directory_pdf,images_output_directory_png,images_output_directory_eps]
    for od in images_output_directories:
        if not os.path.exists(od):
            os.makedirs(od)

    parameters_tex = [0]*number_of_parameters
    
    parameter_chains = {}
    marginal_95_CI_limits = {}
    final_state = []
        
    # load parameters and time taken
    param_matrix = np.loadtxt(chain_file,usecols=range(number_of_parameters)+[4+3*number_of_parameters-1])
    
    time_taken = param_matrix[-1,number_of_parameters]
    
    param_matrix = np.delete(param_matrix,[number_of_parameters],axis=1)
    
    end = param_matrix.shape[0]
    timesteps = range(end)
    if not burn:
        burn = end/4
    
    
    print "Real time taken:", time_taken, "s =", time_taken/60, "min =", time_taken/3600, "hr"
    
    for i in range(number_of_parameters):
        parameters_tex[i] = "$"+fig_labels[i]+"$"
        parameter_chains[parameters[i]] = param_matrix[:,i] # load parameter chains separately
        temp_sorted_marginal = np.sort(param_matrix[:,i])
        marginal_95_CI_limits[parameters[i]] = [temp_sorted_marginal[(end-burn)/40],temp_sorted_marginal[-(end-burn)/40]]
        final_state.append(parameter_chains[parameters[i]][-1])
    del temp_sorted_marginal
        
    print model, protocol, c_seed
    print "Chains loaded"

    medians = np.median(param_matrix,axis=0)
    
    cov_matrix = np.cov(param_matrix.T)

    with open(output_directory+'covariance_matrix.txt','w') as covfile:
        np.savetxt(covfile,cov_matrix)
    
    del param_matrix
        
    bestLogLikelihood = -1e9
    with open(chain_file,'r') as chain_file:
        for line in chain_file:
            temp_array = line.split()
            if temp_array[0] == '#':
                continue
            if float(temp_array[number_of_parameters]) > bestLogLikelihood:
                bestLogLikelihood = float(temp_array[number_of_parameters])
                bestTheta = np.array([float(x) for x in temp_array[:number_of_parameters]])
    
    with open(log_file,'w') as outfile:
        outfile.write("*** MODEL " + str(model) + " - PROTOCOL " + str(protocol) + " - WITH NOISE***\n\n")
        outfile.write("Total time taken by MCMC (including discarded burn): " + str(time_taken) + " secs = "
                      + str(time_taken/60) + " mins = " + str(time_taken/3600) + " hours\n\n")
        outfile.write("Total timesteps = " + str(end) + "\n")
        outfile.write("Burn = " + str(burn) + "\n")
        outfile.write("Total number of plotted timesteps = " + str(end-burn) + "\n")
        outfile.write("Final covariance matrix scaling factor (loga) = " + str(final_loga) + "\n")
        

    
    for i in range(number_of_parameters):
            
        current_parameter = parameters[i]
            
        """ Update output log instead of appending every time """

        with open(log_file,'a') as outfile:
            outfile.write("\nParameter " + current_parameter + ":\n\n")
            outfile.write("True value = " + str(true_parameters[i]) + "\n")
            outfile.write("Mean of entire parameter chain = " + str(np.mean(parameter_chains[current_parameter])) + "\n")
            outfile.write("Mean of chain minus burn = "
                          + str(np.mean(parameter_chains[current_parameter][burn:end])) + "\n")
            outfile.write("Plotted mean / true parameter = "
                          + str(np.mean(parameter_chains[current_parameter][burn:end]) / true_parameters[i]) + "\n")
            outfile.write("Mean of last half of chain = "
                          + str(np.mean(parameter_chains[current_parameter][end/2:end])) + "\n")
            outfile.write("Variance: "+str(cov_matrix[i,i])+"\n")
            outfile.write("Variance / (true value)^2: "+str(cov_matrix[i,i]/true_parameters[i]**2)+"\n")
        
        if i==0:
            zoom = 1600
            fig, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(8,8))
            ax1.hist(parameter_chains[parameters[i]][burn:burn+zoom], color='b', bins=50, normed=True)
            ax1.set_ylim(bottom=0)
            ax1.set_ylabel("Normalized histogram", fontsize=20)

            ax2.plot(parameter_chains[current_parameter][burn:burn+zoom],timesteps[burn:burn+zoom], color='b')
            ax2.set_ylabel('MCMC iteration', fontsize=20)

            if true_lines:
                ax1.axvline(x=true_parameters[i], color='red', linewidth=2.5)  # plot vertical line at true value
                ax2.axvline(x=true_parameters[i], color='red', linewidth=2.5)  # plot vertical line at true value

            plt.gca().invert_yaxis()
            ax2.set_ylim(top=burn)

            plt.xlabel(parameters_tex[i], fontsize=24, labelpad=0)
            plt.xticks(rotation=30)
            fig.tight_layout()

            if current_parameter == r'\sigma':
                current_parameter = 'sigma'

            fig.savefig(images_output_directory_pdf+'model_'+str(model)+'_protocol_'+str(protocol)
                        +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                        +str(c_seed)+'_'+current_parameter+'_zoomed.pdf')
            fig.savefig(images_output_directory_eps+'model_'+str(model)+'_protocol_'+str(protocol)
                        +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                        +str(c_seed)+'_'+current_parameter+'_zoomed.eps')
            fig.savefig(images_output_directory_png+'model_'+str(model)+'_protocol_'+str(protocol)
                        +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                        +str(c_seed)+'_'+current_parameter+'_zoomed.png')
            plt.close()
            
        fig, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(8,8))
        ax1.hist(parameter_chains[parameters[i]][burn:end], color='b', bins=50, normed=True)
        ax1.set_ylim(bottom=0)
        ax1.set_ylabel("Probability density", fontsize=20)

        ax2.plot(parameter_chains[current_parameter][burn:end],timesteps[burn:end], color='b')
        ax2.set_ylabel('Iteration', fontsize=20)
            
        if true_lines:
            ax1.axvline(x=true_parameters[i], color='red', linewidth=2.5)  # plot vertical line at true value
            ax2.axvline(x=true_parameters[i], color='red', linewidth=2.5)  # plot vertical line at true value
                
        plt.gca().invert_yaxis()
        ax2.set_ylim(top=burn)

        plt.xlabel(parameters_tex[i], fontsize=24, labelpad=0)
        plt.xticks(rotation=30)
        fig.tight_layout()
            
        if current_parameter == r'\sigma':
            current_parameter = 'sigma'

        fig.savefig(images_output_directory_pdf+'model_'+str(model)+'_protocol_'+str(protocol)
                    +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                    +str(c_seed)+'_'+current_parameter+'.pdf')
        fig.savefig(images_output_directory_eps+'model_'+str(model)+'_protocol_'+str(protocol)
                    +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                    +str(c_seed)+'_'+current_parameter+'.eps')
        fig.savefig(images_output_directory_png+'model_'+str(model)+'_protocol_'+str(protocol)
                    +'_MCMC_noisy_adaptive_covariance_matrix_thinned_every_10_random_seed_1_c_seed_'
                    +str(c_seed)+'_'+current_parameter+'.png')
        plt.close()

    with open(log_file,'a') as outfile:
        outfile.write("\nTrue values:\n")
        for i in range(number_of_parameters):
            outfile.write(str(true_parameters[i])+" ")
        outfile.write("\n\n")
        outfile.write("Final state:\n")
        for i in range(number_of_parameters):
            outfile.write(str(final_state[i])+" ")
        outfile.write("\n\n")
        outfile.write("Medians:\n")
        for i in range(number_of_parameters):
            outfile.write(str(medians[i])+" ")
        outfile.write("\n\n")
        outfile.write("Best logLikelihood state:\n")
        for i in range(number_of_parameters):
            outfile.write(str(bestTheta[i])+" ")
        outfile.write("\n\n")
        outfile.write("Best logLikelihood: " + str(bestLogLikelihood) + "\n\n")
        outfile.write("95% credible intervals:\n\n")

    
    true_line_color = 'green'
    true_line_width = 2
    

    colormin, colormax = 1e9,0
    
    norm = Normalize(vmin=5,vmax=10)
    
    counting_histograms = 0  # need to plot everything before working out colormap normalization (unless there's a clever way)
    while counting_histograms < 2:
        hidden_labels = []
        axes_dict = {}
        
        A = -1
        
        if ( not subset_indices ): # to plot a scatterplot matrix of a subset of the conductances
            subset_indices = range(number_of_parameters)
            
        number_of_parameters = len(subset_indices)
        
        fig = plt.figure(figsize=(3*number_of_parameters,3*number_of_parameters))
        
        for A in range(number_of_parameters):
            for B in range(A+1):
                i = subset_indices[A]
                j = subset_indices[B]
                x_min = min(parameter_chains[parameters[j]][burn:end])
                x_max = max(parameter_chains[parameters[j]][burn:end])
                y_min = min(parameter_chains[parameters[i]][burn:end])
                y_max = max(parameter_chains[parameters[i]][burn:end])
                string_i = str(i)
                string_j = str(j)
                subplot_position = number_of_parameters*A+B+1
                axij = "ax"+str(A)+str(B) # axis string

                if A==B:
                    axes_dict[axij] = plt.subplot(number_of_parameters,number_of_parameters,subplot_position)

                    n, bins, patches = axes_dict[axij].hist(parameter_chains[parameters[i]][burn:end],
                                                            normed=True,bins=50,color='blue')
                    
                    area = sum(n)
                    
                    y,z=0,0
                    Y,Z=False,False
                    for a in range(len(n)):
                        if y < 0.025:
                            y+=n[a]/area
                        else:
                            if not Y:
                                Y = True
                                y_loc = x_min + (x_max-x_min)/100 * (2*a - 1)
                        if z < 0.025:
                            z+=n[-a-1]/area
                        else:
                            if not Z:
                                Z = True
                                z_loc = x_min + (x_max-x_min)/100 * (2*(len(n)-a) - 1)
                        if Y and Z:
                            break
                            
                    y_loc = marginal_95_CI_limits[parameters[i]][0]
                    z_loc = marginal_95_CI_limits[parameters[i]][1]
                        
                    
                    if counting_histograms == 1:
                        with open(log_file,'a') as outfile:
                            outfile.write(parameters[i] + ": " +str(y_loc)+' '+str(z_loc)+'. Range: '+str(z_loc-y_loc)
                                          +'. Range/true: '+str(1.*(z_loc-y_loc)/true_parameters[i])+'.\n')

                    axes_dict[axij].axvline(x=y_loc,color='red',lw=true_line_width)
                    axes_dict[axij].axvline(x=z_loc,color='red',lw=true_line_width)
                    
                    if i==0:
                        FIG = plt.figure(figsize=(6,4.5))
                        AX = FIG.add_subplot(111)
                        AX.hist(parameter_chains[parameters[i]][burn:end],
                                normed=True,bins=50,color='blue')
                        AX.axvline(x=y_loc,color='red',ls='dashed',lw=true_line_width)
                        AX.axvline(x=z_loc,color='red',ls='dashed',lw=true_line_width)
                        AX.axvline(x=true_parameters[i],color='green',ls='dotted',lw=1+true_line_width)
                        AX.tick_params(axis='both', which='major', labelsize=10)
                        AX.set_xlabel(parameters_tex[i],fontsize=15)
                        AX.set_ylabel('Marginal distribution',fontsize=15)
                        new_tick_locations = [y_loc,true_parameters[i],z_loc]
                        AX2 = AX.twiny()
                        AX2.tick_params(axis='both', which='major', top='off', labelsize=10)                        
                        AX2.set_xticks(new_tick_locations)
                        AX2.set_xticklabels(["%.5f" % z for z in new_tick_locations],color='red')
                        AX2.get_xticklabels()[1].set_color('green')
                        AX2.set_xlim(AX.get_xlim())
                        plt.xticks(rotation=30)
                        FIG.tight_layout()
                        FIG.savefig(images_output_directory_pdf+'95_CI.pdf')
                        plt.close()
                        
                    if true_lines:
                        axes_dict[axij].axvline(x=true_parameters[i],
                                                color=true_line_color,
                                                linewidth=true_line_width) # plot vertical line at true value
                        
                    hidden_labels += axes_dict[axij].get_yticklabels()
                    axes_dict[axij].yaxis.set_major_locator(plt.NullLocator())
                    x_offset = axes_dict[axij].get_xaxis().get_offset_text()
                    y_offset = axes_dict[axij].get_yaxis().get_offset_text()
                elif B==0:
                    # this column only shares x-axis with ax00
                    axes_dict[axij] = plt.subplot(number_of_parameters,number_of_parameters,subplot_position,
                                                  sharex=axes_dict["ax00"])

                    counts, xedges, yedges, Image = plt.hist2d(parameter_chains[parameters[j]][burn:end],
                                                               parameter_chains[parameters[i]][burn:end],
                                                               cmap='hot_r',bins=50,norm=norm)

                    maxcounts = np.amax(counts)
                    if maxcounts > colormax:
                        colormax = maxcounts

                    mincounts = np.amin(counts)
                    if mincounts < colormin:
                        colormin = mincounts

                    if true_lines:
                        axes_dict[axij].axvline(x=true_parameters[j],
                                                color=true_line_color,
                                                linewidth=true_line_width)  # plot vertical line at true value
                        axes_dict[axij].axhline(y=true_parameters[i],
                                                color=true_line_color,
                                                linewidth=true_line_width)  # plot horizontal line at true value
                    axes = axes_dict[axij].axis()
                    axes_dict[axij].set_yticks(np.linspace(axes[2],axes[3],5)[1:4])
                    x_offset = axes_dict[axij].get_xaxis().get_offset_text()
                    y_offset = axes_dict[axij].get_yaxis().get_offset_text()
                    y_offset.set_rotation(90)
                    y_offset.set_position((-0.15,0))
                    axes_dict[axij].set_ylabel(parameters_tex[i],fontsize=50)
                else:
                    axes_dict[axij] = plt.subplot(number_of_parameters,
                                                  number_of_parameters,
                                                  subplot_position,
                                                  sharex=axes_dict["ax"+str(B)+str(B)],
                                                  sharey=axes_dict["ax"+str(A)+"0"])
                    counts, xedges, yedges, Image = plt.hist2d(parameter_chains[parameters[j]][burn:end],
                                                               parameter_chains[parameters[i]][burn:end],
                                                               cmap='hot_r',bins=50,norm=norm)

                    maxcounts = np.amax(counts)
                    if maxcounts > colormax:
                        colormax = maxcounts

                    mincounts = np.amin(counts)
                    if mincounts < colormin:
                        colormin = mincounts

                    if true_lines:
                        axes_dict[axij].axvline(x=true_parameters[j],
                                                color=true_line_color,
                                                linewidth=true_line_width)  # plot vertical line at true value
                        axes_dict[axij].axhline(y=true_parameters[i],
                                                color=true_line_color,
                                                linewidth=true_line_width)  # plot horizontal line at true value
                    hidden_labels += axes_dict[axij].get_yticklabels()
                    x_offset = axes_dict[axij].get_xaxis().get_offset_text()
                    y_offset = axes_dict[axij].get_yaxis().get_offset_text()

                if A==number_of_parameters-1:
                    axes = axes_dict[axij].axis()
                    axes_dict[axij].set_xticks(np.linspace(axes[0],axes[1],5)[1:4])
                    axes_dict[axij].set_xlabel(parameters_tex[j],fontsize=50)
                else:
                    hidden_labels += axes_dict[axij].get_xticklabels()
                    x_offset.set_visible(False)

                if B!=0:
                    y_offset.set_visible(False)

                axes_dict[axij].get_xaxis().get_major_formatter().set_useOffset(False)
                axes_dict[axij].get_yaxis().get_major_formatter().set_useOffset(False)
                plt.xticks(rotation=30)



        axes_dict['ax01'] = plt.subplot(number_of_parameters,number_of_parameters,2)
        axes_dict['ax01'].axis('off')
        plt.setp(hidden_labels, visible=False)
        
        axes_dict['ax02'] = plt.subplot(number_of_parameters,number_of_parameters,3)
        axes_dict['ax02'].axis('off')
        plt.colorbar(ax=axes_dict['ax02'],fraction=0.7,pad=0,aspect=10)
        
        fig.tight_layout()
        if ( len(subset_indices) == number_of_parameters ):
            fig.savefig(images_output_directory_png+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix.png')
            fig.savefig(images_output_directory_pdf+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix.pdf')
            fig.savefig(images_output_directory_eps+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix.eps')
        else:
            print subset_indices
            fig.savefig(images_output_directory_png+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix_subset.png')
            fig.savefig(images_output_directory_pdf+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix_subset.pdf')
            fig.savefig(images_output_directory_eps+'model_'+str(model)+'protocol_'+str(protocol)+'_scatterplot_matrix_subset.eps')
        plt.close()

        norm = Normalize(vmin=colormin,vmax=colormax)
        counting_histograms+=1


try:
    model = int(sys.argv[1])
    protocol = int(sys.argv[2])
    c_seed = int(sys.argv[3])
    AnalyseMCMC(model,protocol,c_seed,override=True)
except:
    print "You'll need to edit the AnalyseMCMC.py file to play with the optional settings."
